"""
Port for Domintell
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy
from domipy.module_directory import get_point_id

class Command(domipy.Message):
    # pylint: disable-msg=R0904
    """
    Base Domintell message
    """

    def __init__(self, module_type=None, serial_number=None, channel=0, command=None, data=None):
        domipy.Message.__init__(self, module_type, serial_number)
        self._command = command
        self._data = data
        self._channel = channel

    def populate(self, serial_number, data_type, data_string):
        pass

    def command(self):

        module_type = self.moduleType
        serial_number = self.serialNumber
        channel = self._channel
        data_type = self._command
        value = self._data

        point_id = get_point_id(module_type, serial_number, channel)

        if data_type is None:
            data_type = ''

        if len(data_type) < 1:
            data_type = ''

        if channel is None or channel == -1:
            # module without channel
            if value is not None:
                return "{:3}{:>6}{}{}".format(module_type, point_id, data_type, value)
            return "{:3}{:6d}{}".format(module_type, point_id, data_type)
        else:
            # normal multichannel module
            if value is not None:
                return "{:3}{:>8}{}{}".format(module_type, point_id, data_type, value)
            return "{:3}{:>8}{}".format(module_type, point_id, data_type)

    def to_string(self):
        """
        Convert to domintell controll string
        This method can be overridden in subclasses to include more than just generic attributes
        :return: str
        """
        return self.command()

    def get_values(self):
        raise NotImplementedError



"""
DPBUx  module status
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
import domipy.messages
from domipy.messages import GenericDIOStatusMessage

BU1_COMMAND_CODE = "BU1"
BU2_COMMAND_CODE = "BU2"
BU4_COMMAND_CODE = "BU4"
BU6_COMMAND_CODE = "BU6"

class DPBU01StatusMessage(GenericDIOStatusMessage):
    """
    DPBU01 module status
    """
    def __init__(self, address=None):
        GenericDIOStatusMessage.__init__(self, 1)
        self.moduleType = BU1_COMMAND_CODE

class DPBU02StatusMessage(DPBU01StatusMessage):
    """
    DPBU02 module status
    """
    def __init__(self, address=None):
        GenericDIOStatusMessage.__init__(self, 2)
        self.moduleType = BU2_COMMAND_CODE

class DPBU04StatusMessage(DPBU02StatusMessage):
    """
    DPBU04 module status
    """
    def __init__(self, address=None):
        GenericDIOStatusMessage.__init__(self, 4)
        self.moduleType = BU4_COMMAND_CODE

class DPBU06StatusMessage(DPBU04StatusMessage):
    """
    DPBU06 module status
    """
    def __init__(self, address=None):
        GenericDIOStatusMessage.__init__(self, 6)
        self.moduleType = BU6_COMMAND_CODE

domipy.register_command(BU1_COMMAND_CODE, DPBU01StatusMessage)
domipy.register_command(BU2_COMMAND_CODE, DPBU02StatusMessage)
domipy.register_command(BU4_COMMAND_CODE, DPBU04StatusMessage)
domipy.register_command(BU6_COMMAND_CODE, DPBU06StatusMessage)



"""
Module Info Message
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

class ModuleInfoMessage(domipy.Message):
    """
    Module information
    """

    def __init__(self, moduleType, data=None):

        domipy.Message.__init__(self)

        self.moduleType = moduleType
        self.serialNumber = data[3:9].strip()
        self.channel = 1

        _pos = ''
        _name_section_start = 9
        if data[9:10] == '-':
            self.channel = data[10:11]
            _pos = '-' + self.channel
            _name_section_start = 11
        
        # point_id is used for compatibility with CornFlower and
        # for sending commands back to domintell bus.
        # point id is a concept widely used in industrial control :)
        self.point_id = self.serialNumber.strip() + _pos

        # user point id as a default name
        self.name = self.point_id

           # start of name section
        _name_section_end = data.find("[")  # end of name section
        _path_section_end = data.find("]")  # path section end

        if _name_section_start > 0:
            if _name_section_end > _name_section_start:
                self.name = data[_name_section_start: _name_section_end].strip()
        else:
            if self.moduleType == 'TRP':
                self.name = data[9:]
        
        if _path_section_end > _name_section_end:
            self.path = data[_name_section_end + 1 : _path_section_end].rstrip('|')


    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        json_dict['module_type'] = self.moduleType
        json_dict['point_id'] = self.point_id
        json_dict['serial_number'] = self.serialNumber
        json_dict['channel'] = self.channel
        json_dict['name'] = self.name
        json_dict['path'] = self.path
        return json.dumps(json_dict)




"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

class ControllMessage(domipy.Message):
    """
    Control message
    Start or End of APPINFO sections
    """

    def __init__(self, message_type=None, data=None):

        domipy.Message.__init__(self, message_type)
        if self.moduleType == "APPINFO":
            self._is_appinfo_mode = True
        elif self.moduleType == "END APPINFO":
            self._is_appinfo_mode = False
        self._message = data
    
    def populate(self, serialNumber, dataType, dataString):
        pass

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        json_dict['is_app_info'] = self._is_appinfo_mode
        json_dict['info_message'] = self._message
        return json.dumps(json_dict)



"""
DO status (to be inherited)
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

DO_COMMAND_CODE = "DO"

class GenericDOStatusMessage(domipy.Message):
    """
    Generic Digital output module status
    """

    def __init__(self, outputCount=1, address=None):
        domipy.Message.__init__(self)
        self.moduleType = DO_COMMAND_CODE
        self.outputCount = outputCount
        self.serialNumber = None
        self.dataType = None
        self.outputs = {}
        for i in range(0, self.outputCount):
            self.outputs[i] = 0

    def populate(self, serialNumber, dataType, dataString):
        """
        :return: None
        """
        # assert isinstance(dataString, str)

        self.serialNumber = serialNumber
        self.dataType = dataType

        mask = int(dataString[0:2].strip(), 16)

        for output in range(0, self.outputCount):
            c = pow(2, output)
            self.outputs[output] = 1 if (mask & c) == c else 0

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        for output in range(0, self.outputCount):
            if output < len(self.outputs):
                json_dict['output{}'.format(output)] = self.outputs[output]
        return json.dumps(json_dict)
    
    def get_values(self):
        return self.outputs
    
    def is_on(self, channel):
        return self.outputs[channel - 1] == 1

    def is_opening(self, channel):
        return self.outputs[channel*2 - 2] == 1

    def is_closing(self, channel):
        return self.outputs[channel*2 - 1] == 1



"""
DBIR01 - 8th bipolar relays controll module 
DTRP01 - 4th trip switched controll module

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy
import domipy.messages

class DBIR01Module(domipy.Module):
    """
    DBIR - relays controll module (8 channels)    
    """
    COMMAND_CODE = 'BIR'

    def __init__(self, serial_number, controller):
        domipy.Module.__init__(self, serial_number, controller)

    def is_on(self, channel):
        if channel < self.number_of_channels():
            if self._values[channel] > 0:
                return True
        return False
    
    def get_value(self, channel):
        if channel < self.number_of_channels():
            return self._values[channel] > 0

    def set_value(self, channel, value):
        if channel < self.number_of_channels():
            message = domipy.SetDigitalOutputMessage(self.get_module_code(), self.get_serial_number(), channel, value)
            self._controller.send(message)

    def togle(self, channel):
        if channel < self.number_of_channels():
            message = domipy.TogleDigitalOutputMessage(self.get_module_code(), self.get_serial_number, channel)
            self._controller.send(message)

    def turn_on(self, channel):
        if channel < self.number_of_channels():
            message = domipy.SetDigitalOutputOnMessage(self.get_module_code(), self.get_serial_number(), channel)
            self._controller.send(message)
 
    def turn_off(self, channel):
        if channel < self.number_of_channels():
            message = domipy.SetDigitalOutputOffMessage(self.get_module_code(), self.get_serial_number(), channel)
            self._controller.send(message)

    def number_of_channels(self):
        return 8

    def _on_message(self, message):
        if isinstance(message, domipy.DBIRStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))

class DTRP01Module(DBIR01Module):
    """
    Controll up to 4 trip switches
    """
    COMMAND_CODE = 'TRP'
    
    def number_of_channels(self):
        return 4

    def _on_message(self, message):
        if isinstance(message, domipy.DTRPStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))

class DTRP02Module(DBIR01Module):
    """
    Controll up to 4 trip switches
    2 x 2shutter command with teleruptors
    Bit 0 Relay 1 = UP
    Bit 1 Relay 1 = DOWN ...
    """
    COMMAND_CODE = 'TPV'
    
    def number_of_channels(self):
        return 4

class DTRV01Module(DBIR01Module):
    """
    DTRV01 Module
    4 shutter inverters
    Bit 0 Relay 1 = UP
    Bit 1 Relay 1 = DOWN ...
    """
    COMMAND_CODE = 'TRV'
    
    def number_of_channels(self):
        return 4
    
    def __init__(self, serial_number, controller):
        domipy.Module.__init__(self, serial_number, controller)

    def is_opening(self, channel):
        if channel < self.number_of_channels():
            if self._values[channel*2] > 0:
                return True
        return False

    def is_closing(self, channel):
        if channel < self.number_of_channels():
            if self._values[channel*2 + 1] > 0:
                return True
        return False

    def is_closed(self, channel):
        return None

    def close_cover(self, channel):
        if channel < self.number_of_channels():
            message = domipy.SetDigitalOutputCloseMessage(self.get_module_code(), self.get_serial_number(), channel)
            self._controller.send(message)

    def open_cover(self, channel):
        if channel < self.number_of_channels():
            message = domipy.SetDigitalOutputOpenMessage(self.get_module_code(), self.get_serial_number(), channel)
            self._controller.send(message)

    def stop_cover(self, channel):
        if channel < self.number_of_channels():
            message = domipy.SetDigitalOutputStopMessage(self.get_module_code(), self.get_serial_number(), channel)
            self._controller.send(message)

    def _on_message(self, message):
        if isinstance(message, domipy.DTRVStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))        

class DTRVBT01Module(DBIR01Module):
    """
    1 DC shutter command
        0 = UP
        1 = DOWN    
    """
    COMMAND_CODE = 'V24'

    def number_of_channels(self):
        return 1

    def _on_message(self, message):
        if isinstance(message, domipy.DTRVBTStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))

class DFAN01Module(DBIR01Module):
    """
    Fan controller
    """
    # TODO I have no DFAN01 module, need to implement it normally
    COMMAND_CODE = 'FAN'

    def number_of_channels(self):
        return 5

class DMR01Module(DBIR01Module):
    """
    5 monopolar relays control module
    """
    COMMAND_CODE = 'DMR'

    def number_of_channels(self):
        return 5

    def _on_message(self, message):
        if isinstance(message, domipy.DDMRStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))


class DLED01Module(DBIR01Module):
    """
    4 LED's driver
    """
    COMMAND_CODE = 'LED'

    def number_of_channels(self):
        return 4

    def _on_message(self, message):
        if isinstance(message, domipy.DLEDStatusMessage):
            self._values = message.get_values()
            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))


domipy.register_module_class(DBIR01Module)
domipy.register_module_class(DTRP01Module)
domipy.register_module_class(DTRP02Module)
domipy.register_module_class(DTRV01Module)
domipy.register_module_class(DTRVBT01Module)
domipy.register_module_class(DMR01Module)
domipy.register_module_class(DLED01Module)



"""
:author: Thomas Delaet <thomas@delaet.org>
"""
import domipy

COMMAND_CODE = "__MODULE_STATUS__"


class ModuleStatusRequest(domipy.Command):
    """
        Request module status
    """
    def __init__(self, moduleType, serialNumber):
        domipy.Command.__init__(self, moduleType, serialNumber)
        self.moduleType = moduleType

    def command(self):
        return "{:3}{:>6}%S".format(self.moduleType, self.serialNumber)
        #return "{:3}{:>6}-4".format(self.moduleType, self.serialNumber)

domipy.register_command(COMMAND_CODE, ModuleStatusRequest)



"""
Dimmer control module DDIM01
Controlls up to 8 dimmers DD500, DD750, DD1000 and DD10V

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy
import domipy.messages


class DDIM01Module(domipy.Module):
    """
    Dimmer control module DDIM01    
    """
    COMMAND_CODE = 'DIM'

    def __init__(self, serial_number, controller):
        domipy.Module.__init__(self, serial_number, controller)

    def is_on(self, channel):
        if channel < self.number_of_channels():
            if self._values[channel] > 0:
                return True
        return False
    
    def get_value(self, channel):
        if channel < self.number_of_channels():
            return self._values[channel]

    def set_value(self, channel, value):
        if channel < self.number_of_channels():
            message = domipy.SetAnalogOutputMessage(self.get_module_code(), self.get_serial_number(), channel, value)
            self._controller.send(message)

    def turn_on(self, channel):
        self.set_value(channel, 100)

    def turn_off(self, channel):
        self.set_value(channel, 0)
    
    def number_of_channels(self):
        return 8

    def _on_message(self, message):
        if isinstance(message, domipy.DDIMStatusMessage):
            # got DDIM status message (contains all channels)
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))

domipy.register_module_class(DDIM01Module)



"""
DIO (Input / Output) status (to be inherited)
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

DIO_COMMAND_CODE = "DIO"

class GenericDIOStatusMessage(domipy.Message):
    """
    Generic Digital input & output hybrid module status
    """

    def __init__(self, pinCount=1, address=None):
        domipy.Message.__init__(self)
        self.moduleType = DIO_COMMAND_CODE
        self.pinCount = pinCount
        self.serialNumber = None
        self.dataType = None
        self._inputs = {}
        self._outputs = {}

        for i in range(0, self.pinCount):
            self._inputs[i] = 0

        for i in range(0, self.pinCount):
            self._outputs[i] = 0
    
    def get_inputs(self):
        return self._inputs

    def get_outputs(self):
        return self._outputs

    def get_input(self, channel):
        if channel < self.pinCount:
            return self._inputs[channel]
        return 0

    def get_output(self, channel):
        if channel < self.pinCount:
            return self._outputs[channel]
        return 0
    
    def is_input(self):
        if self.dataType == 'I':
            return True
        return False

    def is_output(self):
        if self.dataType == 'O':
            return True
        return False

    def populate(self, serialNumber, dataType, dataString):
        """
        :return: None
        """
        assert isinstance(dataString, str)

        self.serialNumber = serialNumber
        self.dataType = dataType

        mask = int(dataString[0:2].strip(), 16)

        if dataType == 'I':
            for input in range(0, self.pinCount):
                self._inputs[input] = 1 if (mask & (input + 1)) else 0

        if dataType == 'O':
            for output in range(0, self.pinCount):
                self._outputs[output] = 1 if (mask & (output + 1)) else 0

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        for input in range(0, self.pinCount):
            if input < len(self._inputs):
                json_dict['input{}'.format(input + 1)] = self._inputs[input]
            if input < len(self._outputs):
                json_dict['output{}'.format(input + 1)] = self._outputs[input]
        return json.dumps(json_dict)



"""
:author: Thomas Delaet <thomas@delaet.org>

Port for Domintell
:author: Zilvinas Binisevicius <zilvinas@binis.me>

"""
import logging
import domipy

NORMAL_MODE = 0
APP_INFO_MODE = 1
MSG_INFO_WRAPPER = "INFO"
MSG_SESSION_OPENED =  'Session opened'
MSG_SESSION_CLOSED =  'Session closed'
MSG_SESSION_TIMEOUT = 'Session timeout'
MSG_LOGIN_WAITING_FOR_LOGINSW = "Waiting for LOGINPSW"
MSG_LOGIN_REQUESTSALT = "REQUESTSALT"
MSG_HELLO_WORLD = "WORLD"

class ParserError(Exception):
    """
    Error when invalid message is received
    """

class DomintellParser(object):
    """
    Transform Domintell message to Message object
    """

    def __init__(self, controller):
        assert isinstance(controller, domipy.Controller)
        self._mode = NORMAL_MODE
        self.logger = logging.getLogger('domintell')
        self.controller = controller

    def feed(self, data):
        message = self.parse(data)
        if isinstance(message, domipy.Message):
            self.controller.new_message(message)

    def parse(self, data):
        """
        parse message and return Message object, or None
        """
        data = data.lstrip('\r\n ').rstrip('\r\n')
        assert len(data) > 0
        assert len(data) >= domipy.MINIMUM_MESSAGE_SIZE

        self.logger.info("Processing message [%s]", data.strip('\r\n '))
        if len(data) > domipy.MAXIMUM_MESSAGE_SIZE:
            self.logger.warning("Domintell message are maximum %s chars, this one is %s", str(
                domipy.MAXIMUM_MESSAGE_SIZE), str(len(data)))
            return
        ## Real parsing here

        message_components = data.split(':')
        message_type = message_components[0]

        if message_type == MSG_INFO_WRAPPER:
            return self.parse_info(message_components)

        module_type = data[0:3]
        serial_number = data[3:9].strip()
        data_type = data[9:10]
        data_string = data[10:].rstrip()

        i = ['INF', '!! ']

        if module_type == 'APP':
            # looks like we received APPINFO start message
            # check more data
            logging.info("Switching to APPINFO moder")
            if data[0:7] == "APPINFO":
                # switching to APPINFO mode
                self._mode = APP_INFO_MODE

                # TODO: Start timer, to reset mode if end packet is lost
                # ..
                return domipy.ControllMessage('APPINFO', data)
        if module_type == 'END':
            # looks like we received APPINFO END message
            # check more data
            if data[0:11] == "END APPINFO":
                self._mode = NORMAL_MODE
                return domipy.ControllMessage('END APPINFO', data)

        
        if self._mode == APP_INFO_MODE:
            # we are in app info mode, regular messages won't be processed
            if self.contains_all(data, '[|]'):
                # module info message
                return domipy.ModuleInfoMessage(module_type, data)
        else:
            # normal mode
            if module_type in i:
                # some info message
                if data[0:24] == MSG_SESSION_OPENED:
                    return domipy.SessionOpenedMessage(data=data)
                elif data[0:24] == MSG_SESSION_CLOSED:
                    return domipy.SessionClosedMessage(data=data)
                elif data[0:25] == MSG_SESSION_TIMEOUT:
                    return domipy.SessionTimeoutMessage(data=data)
                return domipy.InfoMessage(module_type, data)

            # normal message
            if module_type in domipy.CommandRegistry:
                message = domipy.CommandRegistry[module_type]()
                message.populate(serial_number, data_type, data_string)
                return message
            else:
                self.logger.debug("Unrecognized message [%s]", str(module_type))

    def parse_info(self, message_components):
        if message_components[1] == MSG_SESSION_OPENED:
            return domipy.SessionOpenedMessage(data=message_components[1])
        elif message_components[1] == MSG_LOGIN_REQUESTSALT:
            return domipy.LoginRequestSaltMessage(data=message_components)
        elif message_components[1] == MSG_LOGIN_WAITING_FOR_LOGINSW:
            return domipy.WaitingForLoginswMessage(data=message_components)
        elif message_components[1] == MSG_SESSION_CLOSED:
            return domipy.SessionClosedMessage(data=message_components[1])
        elif message_components[1] == MSG_SESSION_TIMEOUT:
            return domipy.SessionTimeoutMessage(data=message_components[1])
        elif message_components[1] == MSG_HELLO_WORLD:
            return domipy.InfoMessage(data=message_components[1])
        return domipy.InfoMessage("INFO", message_components[1])

    def contains_all(self, current_string, current_set):
        return 0 not in [c in current_string for c in current_set]

    def contains_any(self, current_string, current_set):
        return 1 in [c in current_string for c in current_set]

    




"""
Temperature status
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.utils import DecimalEncoder

TEM_COMMAND_CODE = "TEM"
TE1_COMMAND_CODE = "TE1"
TE2_COMMAND_CODE = "TE2"

class GenericTemperaturetatusMessage(domipy.Message):
    """
    Generic Temperature input module status
    """

    def __init__(self, address=None):
        domipy.Message.__init__(self)
        self.moduleType = TEM_COMMAND_CODE
        self.serialNumber = None
        self.dataType = None
        self._current = None
        self._mode = None
        self._set_point = None
        self._range = None

    def get_temperature(self):
        return self._current
    
    def get_mode(self):
        return self._mode

    def get_set_point(self):
        return self._set_point
    
    def get_range(self):
        return self._range

    def populate(self, serialNumber, dataType, dataString):
        """
        :return: None
        """
        # assert isinstance(dataString, str)

        self.serialNumber = serialNumber
        self.dataType = dataType

        data = dataString.split()

        self._current = float(data[0])
        self._set_point = float(data[1])
        self._mode = data[2]
        self._range = float(data[3])

    def to_json(self):
        """
        :return: str
        """
        # FIXME
        json_dict = self.to_json_basic()
        json_dict['current'] = self._current
        json_dict['mode'] = self._mode
        json_dict['set_point'] = self._set_point
        json_dict['range'] = self._range
        return json.dumps(json_dict)


class TE1TemperaturetatusMessage(GenericTemperaturetatusMessage):
    def __init__(self, address=None):
        GenericTemperaturetatusMessage.__init__(self)
        self.moduleType = TE1_COMMAND_CODE

class TE2TemperaturetatusMessage(GenericTemperaturetatusMessage):
    def __init__(self, address=None):
        GenericTemperaturetatusMessage.__init__(self)
        self.moduleType = TE2_COMMAND_CODE


domipy.register_command(TE1_COMMAND_CODE, TE1TemperaturetatusMessage)
domipy.register_command(TE2_COMMAND_CODE, TE2TemperaturetatusMessage)



"""
AO status (to be inherited)
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

AI_COMMAND_CODE = "AI"

class GenericAIStatusMessage(domipy.Message):
    """
    Generic Analog output module status
    """

    def __init__(self, inputCount=1, address=None):
        domipy.Message.__init__(self)
        self.moduleType = AI_COMMAND_CODE
        self.inputCount = inputCount
        self.serialNumber = None
        self.dataType = None
        self.inputs = []
        for i in range(0, self.inputCount):
            self.inputs[i] = 0

    def populate(self, serialNumber, dataType, dataString):
        """
        :return: None
        """
        assert isinstance(dataString, str)

        self.serialNumber = serialNumber
        self.dataType = dataType

        for input in range(0, self.inputCount):
            self.inputs[input] = int(dataString[input * 2: 2].strip(), 16)

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        for input in range(0, self.inputCount):
            if input < len(self.inputs):
                json_dict['input{}'.format(input)] = self.inputs[input]
        return json.dumps(json_dict)



"""
DI status (to be inherited)
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

DI_COMMAND_CODE = "DI"

class GenericDIStatusMessage(domipy.Message):
    """
    Generic Digital input module status
    """

    def __init__(self, moduleType=None, inputCount=1, address=None):
        # assert isinstance(moduleType, str)
        # assert isinstance(inputCount, int) 
        domipy.Message.__init__(self, moduleType)
        self.moduleType = DI_COMMAND_CODE
        self.inputCount = inputCount
        self.serialNumber = None
        self.dataType = None
        self.inputs = {}
        for i in range(0, self.inputCount):
            self.inputs[i] = 0

    def populate(self, serialNumber, dataType, dataString):
        """
        :return: None
        """
        self.serialNumber = serialNumber
        self.dataType = dataType
        mask = int(dataString[0:2].strip(), 16)

        for input in range(0, self.inputCount):
            c = pow(2, input)
            self.inputs[input] = 1 if (mask & c) == c  else 0

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        for input in range(0, self.inputCount):
            if input < len(self.inputs):
                json_dict['input{}'.format(input)] = self.inputs[input]
        return json.dumps(json_dict)

    def get_values(self):
        return self.inputs



"""
DMR module status
Output card to control 8 250V/8A two-pole relays.

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages import GenericDOStatusMessage

DMR_COMMAND_CODE = "DMR"

class DDMRStatusMessage(GenericDOStatusMessage):
    """
    DBIR module status
    """

    def __init__(self, address=None):
        GenericDOStatusMessage.__init__(self, 5)
        self.moduleType = DMR_COMMAND_CODE

domipy.register_command(DMR_COMMAND_CODE, DDMRStatusMessage)
